@@ -24,8 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gdk.h"
-#include "config.h"
 
 #include "gdkwindow.h"
 #include "gdkinputprivate.h"
@@ -96,7 +96,6 @@ gdk_window_impl_fb_init (GdkWindowFBData *impl)
 {
   impl->drawable_data.depth = gdk_display->modeinfo.bits_per_pixel;
   impl->drawable_data.colormap = gdk_colormap_get_system ();
-  impl->event_mask = GDK_STRUCTURE_MASK;
   impl->shape = NULL;
 }
 
@@ -189,6 +188,9 @@ gdk_window_new (GdkWindow     *parent,
 
   private->parent = parent_private;
 
+  private->accept_focus = TRUE;
+  private->focus_on_map = TRUE;
+
   if (attributes_mask & GDK_WA_X)
     x = attributes->x;
   else
@@ -1439,8 +1441,8 @@ gdk_window_set_transient_for (GdkWindow *window,
 }
 
 void
-gdk_window_set_background (GdkWindow *window,
-			   GdkColor  *color)
+gdk_window_set_background (GdkWindow      *window,
+			   const GdkColor *color)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
   
@@ -1469,6 +1471,7 @@ gdk_window_set_back_pixmap (GdkWindow *window,
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (pixmap == NULL || !parent_relative);
+  g_return_if_fail (pixmap == NULL || gdk_drawable_get_depth (window) == gdk_drawable_get_depth (pixmap));
 
   old_pixmap = private->bg_pixmap;
 
@@ -1551,7 +1554,7 @@ gdk_window_get_origin (GdkWindow *window,
 		       gint      *x,
 		       gint      *y)
 {
-  g_return_val_if_fail (window != NULL, 0);
+  g_return_val_if_fail (window != NULL, FALSE);
   
   if (x)
     *x = GDK_DRAWABLE_IMPL_FBDATA (window)->abs_x;
@@ -1621,7 +1624,8 @@ gdk_window_fb_get_visible_region (GdkDrawable *drawable)
 }
 
 GdkWindow *
-_gdk_windowing_window_get_pointer (GdkWindow       *window,
+_gdk_windowing_window_get_pointer (GdkDisplay      *display,
+				   GdkWindow       *window,
 				   gint            *x,
 				   gint            *y,
 				   GdkModifierType *mask)
@@ -1631,26 +1635,18 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
   int winy = 0;
   int x_int, y_int;
   gint shape_dx, shape_dy;
-  GdkModifierType my_mask;
   GdkRegion *shape;
 
   g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
   
-  if (!window)
-    window = _gdk_parent_root;
-  
   gdk_window_get_root_origin (window, &x_int, &y_int);
-  gdk_fb_mouse_get_info (&winx, &winy, &my_mask);
+  gdk_fb_mouse_get_info (&winx, &winy, mask);
 
   winx -= x_int;
   winy -= y_int;
 
-  if (x)
-    *x = winx;
-  if (y)
-    *y = winy;
-  if (mask)
-    *mask = my_mask;
+  *x = winx;
+  *y = winy;
   
   return_val = NULL;
   
@@ -1704,10 +1700,24 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
   return return_val;
 }
 
+void
+_gdk_windowing_get_pointer (GdkDisplay       *display,
+			    GdkScreen       **screen,
+			    gint             *x,
+			    gint             *y,
+			    GdkModifierType  *mask)
+{
+  GdkScreen *default_screen = gdk_display_get_default_screen (display);
+  GdkWindow *root_window = gdk_screen_get_root_window (default_screen);
+  
+  *screen = default_screen;
+  _gdk_windowing_window_get_pointer (display, root_window, x, y, mask);
+}
+
 GdkWindow*
-_gdk_windowing_window_at_pointer (GdkScreen *screen,
-				  gint      *win_x,
-				  gint      *win_y)
+_gdk_windowing_window_at_pointer (GdkDisplay *display,
+				  gint       *win_x,
+				  gint       *win_y)
 {
   gint rx, ry;
   GdkWindow *retval = gdk_window_get_pointer (NULL, win_x, win_y, NULL);
@@ -1715,10 +1725,8 @@ _gdk_windowing_window_at_pointer (GdkScreen *screen,
   if (retval)
     {
       gdk_window_get_origin (retval, &ry, &rx);
-      if (win_x)
-	(*win_x) -= rx;
-      if (win_y)
-	(*win_y) -= ry;
+      (*win_x) -= rx;
+      (*win_y) -= ry;
     }
 
   return retval;
@@ -1733,7 +1741,7 @@ gdk_window_get_events (GdkWindow *window)
   if (GDK_WINDOW_DESTROYED (window))
     return 0;
   else
-    return GDK_WINDOW_IMPL_FBDATA (window)->event_mask;
+    return GDK_WINDOW_OBJECT (window)->event_mask;
 }
 
 void          
@@ -1749,8 +1757,7 @@ gdk_window_set_events (GdkWindow       *window,
       GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK |
       GDK_BUTTON3_MOTION_MASK;
   
-  if (!GDK_WINDOW_DESTROYED (window))
-    GDK_WINDOW_IMPL_FBDATA (window)->event_mask = event_mask;
+  GDK_WINDOW_OBJECT (window)->event_mask = GDK_STRUCTURE_MASK | event_mask;
 }
 
 void
@@ -1959,7 +1966,9 @@ gdk_window_shape_combine_mask (GdkWindow *window,
   _gdk_window_shape_combine_region (window,
 				    (mask == GDK_FB_USE_CHILD_SHAPE)?
 				    GDK_FB_USE_CHILD_SHAPE :
-				    gdk_fb_region_create_from_bitmap (mask),
+				    mask ?
+				    gdk_fb_region_create_from_bitmap (mask) :
+				    NULL,
 				    x, y);
 }
 
@@ -1984,6 +1993,38 @@ gdk_window_set_override_redirect (GdkWindow *window,
   /* N/A */
 }
 
+void
+gdk_window_set_accept_focus (GdkWindow *window,
+			     gboolean accept_focus)
+{
+  GdkWindowObject *private;
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  private = (GdkWindowObject *)window;  
+  
+  accept_focus = accept_focus != FALSE;
+
+  if (private->accept_focus != accept_focus)
+    private->accept_focus = accept_focus;
+}
+
+void
+gdk_window_set_focus_on_map (GdkWindow *window,
+			     gboolean focus_on_map)
+{
+  GdkWindowObject *private;
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  private = (GdkWindowObject *)window;  
+  
+  focus_on_map = focus_on_map != FALSE;
+
+  if (private->focus_on_map != focus_on_map)
+    private->focus_on_map = focus_on_map;
+}
+
 void          
 gdk_window_set_icon (GdkWindow *window, 
 		     GdkWindow *icon_window,
@@ -2153,7 +2194,9 @@ gdk_window_maximize (GdkWindow *window)
 {
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  g_warning ("gdk_window_maximize() not implemented.\n");
+  gdk_synthesize_window_state (window,
+			       0,
+			       GDK_WINDOW_STATE_MAXIMIZED);
 }
 
 void
@@ -2161,7 +2204,9 @@ gdk_window_unmaximize (GdkWindow *window)
 {
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  /*  g_warning ("gdk_window_unmaximize() not implemented.\n");*/
+  gdk_synthesize_window_state (window,
+			       GDK_WINDOW_STATE_MAXIMIZED,
+			       0);
 }
 
 void
@@ -2182,6 +2227,38 @@ gdk_window_deiconify (GdkWindow *window)
   /*  g_warning ("gdk_window_deiconify() not implemented.\n");*/
 }
 
+void
+gdk_window_fullscreen (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  g_warning ("gdk_window_fullscreen() not implemented.\n");
+}
+
+void
+gdk_window_unfullscreen (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  /*  g_warning ("gdk_window_unfullscreen() not implemented.\n");*/
+}
+
+void
+gdk_window_set_keep_above (GdkWindow *window, gboolean setting)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  g_warning ("gdk_window_set_keep_above() not implemented.\n");
+}
+
+void
+gdk_window_set_keep_below (GdkWindow *window, gboolean setting)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  g_warning ("gdk_window_set_keep_below() not implemented.\n");
+}
+
 void
 gdk_window_focus (GdkWindow *window,
                   guint32    timestamp)
@@ -2206,6 +2283,20 @@ gdk_window_set_modal_hint (GdkWindow *window,
   g_return_if_fail (GDK_IS_WINDOW (window));
 }
 
+void
+gdk_window_set_skip_taskbar_hint (GdkWindow *window,
+				  gboolean   skips_taskbar)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+}
+
+void
+gdk_window_set_skip_pager_hint (GdkWindow *window,
+				gboolean   skips_pager)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+}
+
 void
 gdk_window_begin_resize_drag (GdkWindow     *window,
                               GdkWindowEdge  edge,
@@ -2249,9 +2340,10 @@ gdk_window_get_frame_extents (GdkWindow    *window,
 }
 
 GdkWindow*
-gdk_window_foreign_new (GdkNativeWindow anid)
+gdk_window_foreign_new_for_display (GdkDisplay      *display,
+				    GdkNativeWindow  anid)
 {
-  return (GdkWindow*) gdk_drawable_ref (anid);
+  return (GdkWindow*) gdk_drawable_ref ((GdkDrawable *)anid);
 }
 
 GdkWindow*
@@ -2260,3 +2352,20 @@ gdk_window_lookup (GdkNativeWindow anid)
   return (GdkWindow*) (anid);
 }
 
+GdkWindow *
+gdk_window_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
+{
+  g_return_val_if_fail (display == gdk_display_get_default(), NULL);
+
+  return (GdkWindow*) (anid);
+}
+
+void
+gdk_window_enable_synchronized_configure (GdkWindow *window)
+{
+}
+
+void
+gdk_window_configure_finished (GdkWindow *window)
+{
+}
