@@ -1823,10 +1823,31 @@ gdk_window_set_cursor (GdkWindow *window,
 			       hcursor, impl->hcursor));
     }
 
-  /* If the pointer is over our window, set new cursor if given */
-  if (gdk_window_get_pointer(window, NULL, NULL, NULL) == window)
-    if (impl->hcursor != NULL)
-      SetCursor (impl->hcursor);
+  if (impl->hcursor != NULL)
+    {
+      /* If the pointer is over our window, set new cursor */
+      GdkWindow *curr_window = gdk_window_get_pointer (window, NULL, NULL, NULL);
+      if (curr_window == window)
+        SetCursor (impl->hcursor);
+      else
+	{
+	  /* Climb up the tree and find whether our window is the
+	   * first ancestor that has cursor defined, and if so, set
+	   * new cursor.
+	   */
+	  GdkWindowObject *curr_window_obj = GDK_WINDOW_OBJECT (curr_window);
+	  while (curr_window_obj &&
+		 !GDK_WINDOW_IMPL_WIN32 (curr_window_obj->impl)->hcursor)
+	    {
+	      curr_window_obj = curr_window_obj->parent;
+	      if (curr_window_obj == GDK_WINDOW_OBJECT (window))
+		{
+	          SetCursor (impl->hcursor);
+		  break;
+		}
+	    }
+	}
+    }
 
   /* Destroy the previous cursor: Need to make sure it's no longer in
    * use before we destroy it, in case we're not over our window but
@@ -2432,9 +2453,9 @@ pixbuf_to_hicon_normal (GdkWindow *window,
 #endif
 
   if (pixmap)
-    g_object_unref (G_OBJECT (pixmap));
+    g_object_unref (pixmap);
   if (mask)
-    g_object_unref (G_OBJECT (mask));
+    g_object_unref (mask);
 
   return hIcon;
 }
@@ -2842,12 +2863,63 @@ gdk_window_begin_resize_drag (GdkWindow     *window,
                               gint           root_y,
                               guint32        timestamp)
 {
+  WPARAM winedge;
+  
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (GDK_WINDOW_DESTROYED (window))
     return;
 
-  /* XXX: isn't all this default on win32 ... */  
+  /* Tell Windows to start interactively resizing the window by pretending that
+   * the left pointer button was clicked in the suitable edge or corner. This
+   * will only work if the button is down when this function is called, and
+   * will only work with button 1 (left), since Windows only allows window
+   * dragging using the left mouse button */
+  if (button != 1)
+    return;
+  
+  /* Must break the automatic grab that occured when the button was pressed,
+   * otherwise it won't work */
+  gdk_display_pointer_ungrab (gdk_display_get_default (), 0);
+
+  switch (edge)
+    {
+    case GDK_WINDOW_EDGE_NORTH_WEST:
+      winedge = HTTOPLEFT;
+      break;
+
+    case GDK_WINDOW_EDGE_NORTH:
+      winedge = HTTOP;
+      break;
+
+    case GDK_WINDOW_EDGE_NORTH_EAST:
+      winedge = HTTOPRIGHT;
+      break;
+
+    case GDK_WINDOW_EDGE_WEST:
+      winedge = HTLEFT;
+      break;
+
+    case GDK_WINDOW_EDGE_EAST:
+      winedge = HTRIGHT;
+      break;
+
+    case GDK_WINDOW_EDGE_SOUTH_WEST:
+      winedge = HTBOTTOMLEFT;
+      break;
+
+    case GDK_WINDOW_EDGE_SOUTH:
+      winedge = HTBOTTOM;
+      break;
+
+    case GDK_WINDOW_EDGE_SOUTH_EAST:
+    default:
+      winedge = HTBOTTOMRIGHT;
+      break;
+    }
+
+  DefWindowProc (GDK_WINDOW_HWND (window), WM_NCLBUTTONDOWN, winedge,
+      MAKELPARAM (root_x - _gdk_offset_x, root_y - _gdk_offset_y));
 }
 
 void
@@ -2862,7 +2934,20 @@ gdk_window_begin_move_drag (GdkWindow *window,
   if (GDK_WINDOW_DESTROYED (window))
     return;
 
-  /* XXX: isn't all this default on win32 ... */  
+  /* Tell Windows to start interactively moving the window by pretending that
+   * the left pointer button was clicked in the titlebar. This will only work
+   * if the button is down when this function is called, and will only work
+   * with button 1 (left), since Windows only allows window dragging using the
+   * left mouse button */
+  if (button != 1)
+    return;
+  
+  /* Must break the automatic grab that occured when the button was pressed,
+   * otherwise it won't work */
+  gdk_display_pointer_ungrab (gdk_display_get_default (), 0);
+
+  DefWindowProc (GDK_WINDOW_HWND (window), WM_NCLBUTTONDOWN, HTCAPTION,
+      MAKELPARAM (root_x - _gdk_offset_x, root_y - _gdk_offset_y));
 }
 
 
