@@ -1,7 +1,8 @@
 /* GDK - The GIMP Drawing Kit
  * gdkdisplay-x11.c
  * 
- * Copyright 2001 Sun Microsystems Inc. 
+ * Copyright 2001 Sun Microsystems Inc.
+ * Copyright (C) 2004 Nokia Corporation
  *
  * Erwann Chenede <erwann.chenede@sun.com>
  *
@@ -25,6 +26,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <unistd.h>
 
 #include <glib.h>
@@ -89,6 +91,7 @@ static const char *const precache_atoms[] = {
   "_NET_WM_WINDOW_TYPE",
   "_NET_WM_WINDOW_TYPE_NORMAL",
   "_NET_WM_USER_TIME",
+  "_NET_VIRTUAL_ROOTS"
 };
 
 GType
@@ -198,11 +201,14 @@ gdk_display_open (const gchar *display_name)
 
   display_x11->leader_gdk_window = gdk_window_new (GDK_SCREEN_X11 (display_x11->default_screen)->root_window, 
 						   &attr, GDK_WA_X | GDK_WA_Y);
+  (_gdk_x11_window_get_toplevel (display_x11->leader_gdk_window))->is_leader = TRUE;
+
   display_x11->leader_window = GDK_WINDOW_XID (display_x11->leader_gdk_window);
 
   display_x11->leader_window_title_set = FALSE;
 
   display_x11->have_render = GDK_UNKNOWN;
+  display_x11->have_render_with_trapezoids = GDK_UNKNOWN;
 
 #ifdef HAVE_XFIXES
   if (XFixesQueryExtension (display_x11->xdisplay, 
@@ -837,10 +843,30 @@ _gdk_windowing_set_default_display (GdkDisplay *display)
   startup_id = g_getenv ("DESKTOP_STARTUP_ID");
   if (startup_id && *startup_id != '\0')
     {
+      gchar *time_str;
+
       if (!g_utf8_validate (startup_id, -1, NULL))
 	g_warning ("DESKTOP_STARTUP_ID contains invalid UTF-8");
       else
 	display_x11->startup_notification_id = g_strdup (startup_id);
+
+      /* Find the launch time from the startup_id, if it's there.  Newer spec
+       * states that the startup_id is of the form <unique>_TIME<timestamp>
+       */
+      time_str = g_strrstr (startup_id, "_TIME");
+      if (time_str != NULL)
+        {
+	  gulong retval;
+          gchar *end;
+          errno = 0;
+
+          /* Skip past the "_TIME" part */
+          time_str += 5;
+
+          retval = strtoul (time_str, &end, 0);
+          if (end != time_str && errno == 0)
+            display_x11->user_time = retval;
+        }
       
       /* Clear the environment variable so it won't be inherited by
        * child processes and confuse things.  
@@ -914,7 +940,7 @@ broadcast_xmessage   (GdkDisplay   *display,
                      0,
                      CopyFromParent,
                      CopyFromParent,
-                     CopyFromParent,
+                     (Visual *)CopyFromParent,
                      CWOverrideRedirect | CWEventMask,
                      &attrs);
   }
@@ -1075,3 +1101,86 @@ gboolean gdk_display_request_selection_notification  (GdkDisplay *display,
 #endif
     return FALSE;
 }
+
+/**
+ * gdk_display_supports_clipboard_persistence
+ * @display: a #GdkDisplay
+ *
+ * Returns whether the speicifed display supports clipboard
+ * persistance; i.e. if it's possible to store the clipboard data after an
+ * application has quit. On X11 this checks if a clipboard daemon is
+ * running.
+ *
+ * Returns: %TRUE if the display supports clipboard persistance.
+ *
+ * Since: 2.6
+ */
+gboolean
+gdk_display_supports_clipboard_persistence (GdkDisplay *display)
+{
+  /* It might make sense to cache this */
+  return XGetSelectionOwner (GDK_DISPLAY_X11 (display)->xdisplay,
+			     gdk_x11_get_xatom_by_name_for_display (display, "CLIPBOARD_MANAGER")) != None;
+}
+
+/**
+ * gdk_display_store_clipboard
+ * @display:          a #GdkDisplay
+ * @clipboard_window: a #GdkWindow belonging to the clipboard owner
+ * @time_:            a timestamp
+ * @targets:	      an array of targets that should be saved, or %NULL 
+ *                    if all available targets should be saved.
+ * @n_targets:        length of the @targets array
+ *
+ * Issues a request to the clipboard manager to store the
+ * clipboard data. On X11, this is a special program that works
+ * according to the freedesktop clipboard specification, available at
+ * <ulink url="http://www.freedesktop.org/Standards/clipboard-manager-spec">
+ * http://www.freedesktop.org/Standards/clipboard-manager-spec</ulink>.
+ *
+ * Since: 2.6
+ */
+void
+gdk_display_store_clipboard (GdkDisplay *display,
+			     GdkWindow  *clipboard_window,
+			     guint32     time_,
+			     GdkAtom    *targets,
+			     gint        n_targets)
+{
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
+  Atom clipboard_manager, save_targets;
+  
+  clipboard_manager = gdk_x11_get_xatom_by_name_for_display (display, "CLIPBOARD_MANAGER");
+  save_targets = gdk_x11_get_xatom_by_name_for_display (display, "SAVE_TARGETS");
+
+  gdk_error_trap_push ();
+
+  if (XGetSelectionOwner (display_x11->xdisplay, clipboard_manager) != None)
+    {
+      Atom property_name = None;
+      Atom *xatoms;
+      int i;
+      
+      if (n_targets > 0)
+	{
+	  property_name = gdk_x11_atom_to_xatom_for_display (display, _gdk_selection_property);
+
+	  xatoms = g_new (Atom, n_targets);
+	  for (i = 0; i < n_targets; i++)
+	    xatoms[i] = gdk_x11_atom_to_xatom_for_display (display, targets[i]);
+
+	  XChangeProperty (display_x11->xdisplay, GDK_WINDOW_XID (clipboard_window),
+			   property_name, XA_ATOM,
+			   32, PropModeReplace, (guchar *)xatoms, n_targets);
+	  g_free (xatoms);
+
+	}
+      
+      XConvertSelection (display_x11->xdisplay,
+			 clipboard_manager, save_targets, property_name,
+			 GDK_WINDOW_XID (clipboard_window), time_);
+      
+    }
+  gdk_error_trap_pop ();
+
+}
