@@ -26,6 +26,7 @@
 
 #include <config.h>
 
+#include "gdkalias.h"
 #include "gdk.h"
 #include "gdkprivate-x11.h"
 #include "gdkinternals.h"
@@ -460,17 +461,22 @@ gdk_check_wm_desktop_changed (GdkWindow *window)
 
   if (toplevel->have_sticky)
     {
+      guchar *data;
       gulong *desktop;
       
+      type = None;
+      gdk_error_trap_push ();
       XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), 
 			  GDK_WINDOW_XID (window),
                           gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_DESKTOP"),
 			  0, G_MAXLONG, False, XA_CARDINAL, &type, 
 			  &format, &nitems,
-                          &bytes_after, (guchar **)&desktop);
+                          &bytes_after, &data);
+      gdk_error_trap_pop ();
 
       if (type != None)
         {
+	  desktop = (gulong *)data;
           toplevel->on_all_desktops = (*desktop == 0xFFFFFFFF);
           XFree (desktop);
         }
@@ -491,6 +497,7 @@ gdk_check_wm_state_changed (GdkWindow *window)
   gint format;
   gulong nitems;
   gulong bytes_after;
+  guchar *data;
   Atom *atoms = NULL;
   gulong i;
 
@@ -501,10 +508,13 @@ gdk_check_wm_state_changed (GdkWindow *window)
   toplevel->have_maxhorz = FALSE;
   toplevel->have_fullscreen = FALSE;
 
+  type = None;
+  gdk_error_trap_push ();
   XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (window),
 		      gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_STATE"),
 		      0, G_MAXLONG, False, XA_ATOM, &type, &format, &nitems,
-		      &bytes_after, (guchar **)&atoms);
+		      &bytes_after, &data);
+  gdk_error_trap_pop ();
 
   if (type != None)
     {
@@ -513,6 +523,8 @@ gdk_check_wm_state_changed (GdkWindow *window)
       Atom maxhorz_atom	= gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_STATE_MAXIMIZED_HORZ");
       Atom fullscreen_atom = gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_STATE_FULLSCREEN");
 
+      atoms = (Atom *)data;
+
       i = 0;
       while (i < nitems)
         {
@@ -2032,8 +2044,14 @@ gdk_wm_protocols_filter (GdkXEvent *xev,
 {
   XEvent *xevent = (XEvent *)xev;
   GdkWindow *win = event->any.window;
-  GdkDisplay *display = GDK_WINDOW_DISPLAY (win);
-  Atom atom = (Atom)xevent->xclient.data.l[0];
+  GdkDisplay *display;
+  Atom atom;
+
+  if (!win)
+      return GDK_FILTER_REMOVE;    
+
+  display = GDK_WINDOW_DISPLAY (win);
+  atom = (Atom)xevent->xclient.data.l[0];
 
   if (atom == gdk_x11_get_xatom_by_name_for_display (display, "WM_DELETE_WINDOW"))
     {
@@ -2051,6 +2069,8 @@ gdk_wm_protocols_filter (GdkXEvent *xev,
       
       event->any.type = GDK_DELETE;
 
+      gdk_x11_window_set_user_time (win, xevent->xclient.data.l[1]);
+
       return GDK_FILTER_TRANSLATE;
     }
   else if (atom == gdk_x11_get_xatom_by_name_for_display (display, "WM_TAKE_FOCUS"))
@@ -2248,7 +2268,7 @@ gdk_event_send_client_message_for_display (GdkDisplay     *display,
 
 
 /* Sends a ClientMessage to all toplevel client windows */
-gboolean
+static gboolean
 gdk_event_send_client_message_to_all_recurse (GdkDisplay *display,
 					      XEvent     *xev, 
 					      guint32     xid,
@@ -2447,6 +2467,7 @@ fetch_net_wm_check_window (GdkScreen *screen)
   gint format;
   gulong n_items;
   gulong bytes_after;
+  guchar *data;
   Window *xwindow;
   
   /* This function is very slow on every call if you are not running a
@@ -2466,11 +2487,13 @@ fetch_net_wm_check_window (GdkScreen *screen)
   XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), screen_x11->xroot_window,
 		      gdk_x11_get_xatom_by_name_for_display (display, "_NET_SUPPORTING_WM_CHECK"),
 		      0, G_MAXLONG, False, XA_WINDOW, &type, &format, 
-		      &n_items, &bytes_after, (guchar **) & xwindow);
+		      &n_items, &bytes_after, &data);
   
   if (type != XA_WINDOW)
     return;
 
+  xwindow = (Window *)data;
+
   gdk_error_trap_push ();
   
   /* Find out if this WM goes away, so we can reset everything. */
@@ -2696,6 +2719,7 @@ static struct
   { "Gtk/ToolbarIconSize", "gtk-toolbar-icon-size" },
   { "Gtk/IMPreeditStyle", "gtk-im-preedit-style" },
   { "Gtk/IMStatusStyle", "gtk-im-status-style" },
+  { "Gtk/Modules", "gtk-modules" },
   { "Gtk/FileChooserBackend", "gtk-file-chooser-backend" },
   { "Gtk/ButtonImages", "gtk-button-images" },
   { "Gtk/MenuImages", "gtk-menu-images" },
