@@ -27,8 +27,10 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <string.h>
 
+#include "gtkalias.h"
 #include "gtkaccellabel.h"
 #include "gtkaccelmap.h"
 #include "gtkmain.h"
@@ -318,25 +320,36 @@ gtk_accel_label_expose_event (GtkWidget      *widget,
 	{
 	  PangoLayout *label_layout;
 	  PangoLayout *accel_layout;
+	  GtkLabel *label = GTK_LABEL (widget);
 
 	  gint x;
 	  gint y;
 	  
+	  label_layout = gtk_label_get_layout (GTK_LABEL (accel_label));
+
 	  if (direction == GTK_TEXT_DIR_RTL)
 	    widget->allocation.x += ac_width;
 	  widget->allocation.width -= ac_width;
+	  if (gtk_label_get_ellipsize (label))
+	    pango_layout_set_width (label_layout,
+				    pango_layout_get_width (label_layout) 
+				    - ac_width * PANGO_SCALE);
+	  
 	  if (GTK_WIDGET_CLASS (parent_class)->expose_event)
 	    GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
 	  if (direction == GTK_TEXT_DIR_RTL)
 	    widget->allocation.x -= ac_width;
 	  widget->allocation.width += ac_width;
+	  if (gtk_label_get_ellipsize (label))
+	    pango_layout_set_width (label_layout,
+				    pango_layout_get_width (label_layout) 
+				    + ac_width * PANGO_SCALE);
 	  
 	  if (direction == GTK_TEXT_DIR_RTL)
 	    x = widget->allocation.x + misc->xpad;
 	  else
 	    x = widget->allocation.x + widget->allocation.width - misc->xpad - ac_width;
 
-	  label_layout = gtk_label_get_layout (GTK_LABEL (accel_label));
 	  gtk_label_get_layout_offsets (GTK_LABEL (accel_label), NULL, &y);
 
 	  accel_layout = gtk_widget_create_pango_layout (widget, gtk_accel_label_get_string (accel_label));
@@ -503,6 +516,84 @@ gtk_accel_label_get_string (GtkAccelLabel *accel_label)
   return accel_label->accel_string;
 }
 
+/* Underscores in key names are better displayed as spaces
+ * E.g., Page_Up should be "Page Up"
+ */
+static void
+substitute_underscores (char *str)
+{
+  char *p;
+
+  for (p = str; *p; p++)
+    if (*p == '_')
+      *p = ' ';
+}
+
+gchar *
+_gtk_accel_label_class_get_accelerator_label (GtkAccelLabelClass *klass,
+					      guint               accelerator_key,
+					      GdkModifierType     accelerator_mods)
+{
+  GString *gstring;
+  gboolean seen_mod = FALSE;
+  gunichar ch;
+  
+  gstring = g_string_new ("");
+  
+  if (accelerator_mods & GDK_SHIFT_MASK)
+    {
+      g_string_append (gstring, klass->mod_name_shift);
+      seen_mod = TRUE;
+    }
+  if (accelerator_mods & GDK_CONTROL_MASK)
+    {
+      if (seen_mod)
+	g_string_append (gstring, klass->mod_separator);
+      g_string_append (gstring, klass->mod_name_control);
+      seen_mod = TRUE;
+    }
+  if (accelerator_mods & GDK_MOD1_MASK)
+    {
+      if (seen_mod)
+	g_string_append (gstring, klass->mod_separator);
+      g_string_append (gstring, klass->mod_name_alt);
+      seen_mod = TRUE;
+    }
+  if (seen_mod)
+    g_string_append (gstring, klass->mod_separator);
+  
+  ch = gdk_keyval_to_unicode (accelerator_key);
+  if (ch && (g_unichar_isgraph (ch) || ch == ' ') &&
+      (ch < 0x80 || klass->latin1_to_char))
+    {
+      switch (ch)
+	{
+	case ' ':
+	  g_string_append (gstring, "Space");
+	  break;
+	case '\\':
+	  g_string_append (gstring, "Backslash");
+	  break;
+	default:
+	  g_string_append_unichar (gstring, g_unichar_toupper (ch));
+	  break;
+	}
+    }
+  else
+    {
+      gchar *tmp;
+      
+      tmp = gtk_accelerator_name (accelerator_key, 0);
+      if (tmp[0] != 0 && tmp[1] == 0)
+	tmp[0] = g_ascii_toupper (tmp[0]);
+      substitute_underscores (tmp);
+      g_string_append (gstring, tmp);
+      g_free (tmp);
+    }
+
+  return g_string_free (gstring, FALSE);
+}
+
 gboolean
 gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 {
@@ -524,65 +615,15 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 
       if (key && key->accel_flags & GTK_ACCEL_VISIBLE)
 	{
-	  GString *gstring;
-	  gboolean seen_mod = FALSE;
-	  gunichar ch;
-	  
-	  gstring = g_string_new (accel_label->accel_string);
-	  g_string_append (gstring, gstring->len ? class->accel_seperator : "   ");
-	  
-	  if (key->accel_mods & GDK_SHIFT_MASK)
-	    {
-	      g_string_append (gstring, class->mod_name_shift);
-	      seen_mod = TRUE;
-	    }
-	  if (key->accel_mods & GDK_CONTROL_MASK)
-	    {
-	      if (seen_mod)
-		g_string_append (gstring, class->mod_separator);
-	      g_string_append (gstring, class->mod_name_control);
-	      seen_mod = TRUE;
-	    }
-	  if (key->accel_mods & GDK_MOD1_MASK)
-	    {
-	      if (seen_mod)
-		g_string_append (gstring, class->mod_separator);
-	      g_string_append (gstring, class->mod_name_alt);
-	      seen_mod = TRUE;
-	    }
-	  if (seen_mod)
-	    g_string_append (gstring, class->mod_separator);
+	  GtkAccelLabelClass *klass;
+	  gchar *tmp;
 
-	  ch = gdk_keyval_to_unicode (key->accel_key);
-	  if (ch && (g_unichar_isgraph (ch) || ch == ' ') &&
-	      (ch < 0x80 || class->latin1_to_char))
-	    {
-	      switch (ch)
-		{
-		case ' ':
-		  g_string_append (gstring, "Space");
-		  break;
-		case '\\':
-		  g_string_append (gstring, "Backslash");
-		  break;
-		default:
-		  g_string_append_unichar (gstring, g_unichar_toupper (ch));
-		  break;
-		}
-	    }
-	  else
-	    {
-	      gchar *tmp;
-	      
-	      tmp = gtk_accelerator_name (key->accel_key, 0);
-	      if (tmp[0] != 0 && tmp[1] == 0)
-		tmp[0] = g_ascii_toupper (tmp[0]);
-	      g_string_append (gstring, tmp);
-	      g_free (tmp);
-	    }
-	  g_free (accel_label->accel_string);
-	  accel_label->accel_string = gstring->str;
-	  g_string_free (gstring, FALSE);
+	  klass = GTK_ACCEL_LABEL_GET_CLASS (accel_label);
+	  tmp = _gtk_accel_label_class_get_accelerator_label (klass,
+							      key->accel_key,
+							      key->accel_mods);
+	  accel_label->accel_string = g_strconcat ("   ", tmp, NULL);
+	  g_free (tmp);
 	}
       if (!accel_label->accel_string)
 	accel_label->accel_string = g_strdup ("-/-");
