@@ -30,6 +30,7 @@
 
 #include <config.h>
 
+#include "gtkalias.h"
 #include "gtkcolorbutton.h"
 #include "gdk/gdkkeysyms.h"
 #include "gdk-pixbuf/gdk-pixbuf.h"
@@ -612,7 +613,7 @@ gtk_color_button_finalize (GObject *object)
   GtkColorButton *color_button = GTK_COLOR_BUTTON (object);
 
   if (color_button->priv->gc != NULL)
-    g_object_unref (G_OBJECT (color_button->priv->gc));
+    g_object_unref (color_button->priv->gc);
   color_button->priv->gc = NULL;
 
   if (color_button->priv->cs_dialog != NULL)
@@ -660,7 +661,7 @@ gtk_color_button_new (void)
  * Since: 2.4
  */
 GtkWidget *
-gtk_color_button_new_with_color (GdkColor *color)
+gtk_color_button_new_with_color (const GdkColor *color)
 {
   return g_object_new (GTK_TYPE_COLOR_BUTTON, "color", color, NULL);
 }
@@ -776,7 +777,7 @@ gtk_color_button_clicked (GtkButton *button)
  **/
 void
 gtk_color_button_set_color (GtkColorButton *color_button,
-			    GdkColor       *color)
+			    const GdkColor *color)
 {
   g_return_if_fail (GTK_IS_COLOR_BUTTON (color_button));
 
