@@ -23,7 +23,9 @@
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
+#include <config.h>
 #include <glib.h>
+#include "gtkalias.h"
 #include "gtkcolorseldialog.h"
 #include "gtkframe.h"
 #include "gtkhbbox.h"
@@ -42,26 +44,29 @@ static GtkWindowClass *color_selection_dialog_parent_class = NULL;
 /* GtkColorSelectionDialog */
 /***************************/
 
-GtkType
+GType
 gtk_color_selection_dialog_get_type (void)
 {
-  static GtkType color_selection_dialog_type = 0;
+  static GType color_selection_dialog_type = 0;
   
   if (!color_selection_dialog_type)
     {
-      GtkTypeInfo colorsel_diag_info =
+      static const GTypeInfo colorsel_diag_info =
       {
-	"GtkColorSelectionDialog",
-	sizeof (GtkColorSelectionDialog),
 	sizeof (GtkColorSelectionDialogClass),
-	(GtkClassInitFunc) gtk_color_selection_dialog_class_init,
-	(GtkObjectInitFunc) gtk_color_selection_dialog_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_color_selection_dialog_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkColorSelectionDialog),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_color_selection_dialog_init,
       };
       
-      color_selection_dialog_type = gtk_type_unique (GTK_TYPE_DIALOG, &colorsel_diag_info);
+      color_selection_dialog_type =
+	g_type_register_static (GTK_TYPE_DIALOG, "GtkColorSelectionDialog",
+				&colorsel_diag_info, 0);
     }
   
   return color_selection_dialog_type;
@@ -70,11 +75,7 @@ gtk_color_selection_dialog_get_type (void)
 static void
 gtk_color_selection_dialog_class_init (GtkColorSelectionDialogClass *klass)
 {
-  GtkObjectClass *object_class;
-  
-  object_class = (GtkObjectClass*) klass;
-  
-  color_selection_dialog_parent_class = gtk_type_class (GTK_TYPE_DIALOG);
+  color_selection_dialog_parent_class = g_type_class_peek_parent (klass);
 }
 
 static void
@@ -111,6 +112,15 @@ gtk_color_selection_dialog_init (GtkColorSelectionDialog *colorseldiag)
                                                      GTK_RESPONSE_HELP);
 
   gtk_widget_hide (colorseldiag->help_button);
+
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (colorseldiag),
+					   GTK_RESPONSE_OK,
+					   GTK_RESPONSE_CANCEL,
+					   GTK_RESPONSE_HELP,
+					   -1);
+
+  gtk_window_set_title (GTK_WINDOW (colorseldiag),
+                        _("Color Selection"));
 }
 
 GtkWidget*
@@ -118,9 +128,13 @@ gtk_color_selection_dialog_new (const gchar *title)
 {
   GtkColorSelectionDialog *colorseldiag;
   
-  colorseldiag = gtk_type_new (GTK_TYPE_COLOR_SELECTION_DIALOG);
-  gtk_window_set_title (GTK_WINDOW (colorseldiag), title);
-  gtk_window_set_policy(GTK_WINDOW (colorseldiag), FALSE, FALSE, TRUE);
+  colorseldiag = g_object_new (GTK_TYPE_COLOR_SELECTION_DIALOG, NULL);
+
+  if (title)
+    gtk_window_set_title (GTK_WINDOW (colorseldiag), title);
+
+  gtk_window_set_resizable (GTK_WINDOW (colorseldiag), FALSE);
   
   return GTK_WIDGET (colorseldiag);
 }
+
