@@ -345,10 +345,16 @@ gtk_dialog_map (GtkWidget *widget)
   if (!window->focus_widget)
     {
       GList *children, *tmp_list;
+      GtkWidget *first_focus = NULL;
       
       do 
 	{
 	  g_signal_emit_by_name (window, "move_focus", GTK_DIR_TAB_FORWARD);
+
+	  if (first_focus == NULL)
+	    first_focus = window->focus_widget;
+	  else if (first_focus == window->focus_widget)
+	    break;
 	}
       while (GTK_IS_LABEL (window->focus_widget));
 
@@ -967,6 +973,12 @@ run_destroy_handler (GtkDialog *dialog, gpointer data)
  *   gtk_widget_destroy (dialog);
  * </programlisting></informalexample>
  * 
+ * Note that even though the recursive main loop gives the effect of a
+ * modal dialog (it prevents the user from interacting with other 
+ * windows while the dialog is run), callbacks such as timeouts, 
+ * IO channel watches, DND drops, etc, <emphasis>will</emphasis> be 
+ * triggered during a gtk_dialog_run() call.
+ * 
  * Return value: response ID
  **/
 gint
