@@ -25,6 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include "gtkalias.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtkplug.h"
@@ -78,7 +79,7 @@ enum {
 static guint plug_signals[LAST_SIGNAL] = { 0 };
 
 GType
-gtk_plug_get_type ()
+gtk_plug_get_type (void)
 {
   static GType plug_type = 0;
 
@@ -561,6 +562,8 @@ gtk_plug_realize (GtkWidget *widget)
 
   widget->style = gtk_style_attach (widget->style, widget->window);
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
+
+  gdk_window_enable_synchronized_configure (widget->window);
 }
 
 static void
