@@ -1,8 +1,12 @@
 #ifndef GTK_TEXT_TAG_H
 #define GTK_TEXT_TAG_H
 
-#include <gtk/gtkobject.h>
+#include <glib-object.h>
 #include <gdk/gdk.h>
+#include <gtk/gtkenums.h>
+
+/* Not needed, retained for compatibility -Yosh */
+#include <gtk/gtkobject.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -112,6 +116,7 @@ typedef struct _GtkTextAppearance GtkTextAppearance;
 
 struct _GtkTextAppearance
 {
+  /*< public >*/
   GdkColor bg_color;
   GdkColor fg_color;
   GdkBitmap *bg_stipple;
@@ -120,11 +125,13 @@ struct _GtkTextAppearance
   /* super/subscript rise, can be negative */
   gint rise;
 
+  /*< private >*/
   /* I'm not sure this can really be used without breaking some things
    * an app might do :-/
    */
   gpointer padding1;
-  
+
+  /*< public >*/
   guint underline : 4;          /* PangoUnderline */
   guint strikethrough : 1;
 
@@ -142,6 +149,7 @@ struct _GtkTextAppearance
   guint inside_selection : 1;
   guint is_text : 1;
 
+  /*< private >*/
   guint pad1 : 1;
   guint pad2 : 1;
   guint pad3 : 1;
@@ -150,8 +158,10 @@ struct _GtkTextAppearance
 
 struct _GtkTextAttributes
 {
+  /*< private >*/
   guint refcount;
 
+  /*< public >*/
   GtkTextAppearance appearance;
 
   GtkJustification justification;
@@ -183,11 +193,13 @@ struct _GtkTextAttributes
 
   PangoLanguage *language;
 
+  /*< private >*/
   /* I'm not sure this can really be used without breaking some things
    * an app might do :-/
    */
   gpointer padding1;
-  
+
+  /*< public >*/
   /* hide the text  */
   guint invisible : 1;
 
@@ -202,6 +214,7 @@ struct _GtkTextAttributes
   /* colors are allocated etc. */
   guint realized : 1;
 
+  /*< private >*/
   guint pad1 : 1;
   guint pad2 : 1;
   guint pad3 : 1;
@@ -215,7 +228,7 @@ void               gtk_text_attributes_copy_values (GtkTextAttributes *src,
 void               gtk_text_attributes_unref       (GtkTextAttributes *values);
 void               gtk_text_attributes_ref         (GtkTextAttributes *values);
 
-GType              gtk_text_attributes_get_type    (void);
+GType              gtk_text_attributes_get_type    (void) G_GNUC_CONST;
 
 
 #ifdef __cplusplus
