@@ -221,6 +221,8 @@ gtk_debug_flag_get_type (void)
       { GTK_DEBUG_KEYBINDINGS, "GTK_DEBUG_KEYBINDINGS", "keybindings" },
       { GTK_DEBUG_MULTIHEAD, "GTK_DEBUG_MULTIHEAD", "multihead" },
       { GTK_DEBUG_MODULES, "GTK_DEBUG_MODULES", "modules" },
+      { GTK_DEBUG_GEOMETRY, "GTK_DEBUG_GEOMETRY", "geometry" },
+      { GTK_DEBUG_ICONTHEME, "GTK_DEBUG_ICONTHEME", "icontheme" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GtkDebugFlag", values);
@@ -1165,6 +1167,7 @@ gtk_image_type_get_type (void)
       { GTK_IMAGE_STOCK, "GTK_IMAGE_STOCK", "stock" },
       { GTK_IMAGE_ICON_SET, "GTK_IMAGE_ICON_SET", "icon-set" },
       { GTK_IMAGE_ANIMATION, "GTK_IMAGE_ANIMATION", "animation" },
+      { GTK_IMAGE_ICON_NAME, "GTK_IMAGE_ICON_NAME", "icon-name" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GtkImageType", values);
