@@ -117,8 +117,8 @@ set_visible_func(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
 
   gtk_tree_model_get (model, iter, 0, &tool_item, -1);
 
-  g_object_get (G_OBJECT (tool_item), "visible", &visible, NULL);
-  g_object_set (G_OBJECT (cell), "active", visible, NULL);
+  g_object_get (tool_item, "visible", &visible, NULL);
+  g_object_set (cell, "active", visible, NULL);
   g_object_unref (tool_item);
 }
 
@@ -135,8 +135,8 @@ visibile_toggled(GtkCellRendererToggle *cell, const gchar *path_str,
   gtk_tree_model_get_iter (model, &iter, path);
 
   gtk_tree_model_get (model, &iter, 0, &tool_item, -1);
-  g_object_get (G_OBJECT (tool_item), "visible", &visible, NULL);
-  g_object_set (G_OBJECT (tool_item), "visible", !visible, NULL);
+  g_object_get (tool_item, "visible", &visible, NULL);
+  g_object_set (tool_item, "visible", !visible, NULL);
   g_object_unref (tool_item);
 
   gtk_tree_model_row_changed (model, path, &iter);
@@ -151,7 +151,7 @@ set_expand_func(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
 
   gtk_tree_model_get (model, iter, 0, &tool_item, -1);
 
-  g_object_set (G_OBJECT (cell), "active", gtk_tool_item_get_expand (tool_item), NULL);
+  g_object_set (cell, "active", gtk_tool_item_get_expand (tool_item), NULL);
   g_object_unref (tool_item);
 }
 
@@ -182,7 +182,7 @@ set_homogeneous_func(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
 
   gtk_tree_model_get (model, iter, 0, &tool_item, -1);
 
-  g_object_set (G_OBJECT (cell), "active", gtk_tool_item_get_homogeneous (tool_item), NULL);
+  g_object_set (cell, "active", gtk_tool_item_get_homogeneous (tool_item), NULL);
   g_object_unref (tool_item);
 }
 
@@ -214,7 +214,7 @@ set_important_func(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
 
   gtk_tree_model_get (model, iter, 0, &tool_item, -1);
 
-  g_object_set (G_OBJECT (cell), "active", gtk_tool_item_get_is_important (tool_item), NULL);
+  g_object_set (cell, "active", gtk_tool_item_get_is_important (tool_item), NULL);
   g_object_unref (tool_item);
 }
 
@@ -469,13 +469,25 @@ toolbar_drag_leave (GtkToolbar     *toolbar,
 {
   if (drag_item)
     {
-      g_object_unref (G_OBJECT (drag_item));
+      g_object_unref (drag_item);
       drag_item = NULL;
     }
   
   gtk_toolbar_set_drop_highlight_item (toolbar, NULL, 0);
 }
 
+static gboolean
+timeout_cb (GtkWidget *widget)
+{
+  static gboolean sensitive = TRUE;
+  
+  sensitive = !sensitive;
+  
+  gtk_widget_set_sensitive (widget, sensitive);
+  
+  return TRUE;
+}
+
 gint
 main (gint argc, gchar **argv)
 {
@@ -587,6 +599,7 @@ main (gint argc, gchar **argv)
   gtk_tool_button_set_label (GTK_TOOL_BUTTON (item), NULL);
   add_item_to_list (store, item, "New");
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+  g_timeout_add (3000, (GSourceFunc) timeout_cb, item);
   gtk_tool_item_set_expand (item, TRUE);
 
   menu = gtk_menu_new ();
@@ -601,7 +614,7 @@ main (gint argc, gchar **argv)
   gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (item), menu);
   add_item_to_list (store, item, "Open");
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
-
+ 
   item = gtk_separator_tool_item_new ();
   add_item_to_list (store, item, "-----");    
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
